﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using System.Drawing;
using System.IO;
using System.Transactions;
using System.Windows.Forms;

namespace EventMediaSpatialApp
{
	public partial class PhotoSearchForm : Form
	{
		private const string ConnStr =
				"Data Source=.;Integrated Security=True;Initial Catalog=EventLibrary;";

		public PhotoSearchForm()
		{
			InitializeComponent();
		}

		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);

			this.LoadRegions();
		}

		private void lnkSearch_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			this.FindRegionPhotos();
		}

		private void lstPhotos_SelectedIndexChanged(object sender, EventArgs e)
		{
			this.DisplayPhoto();
		}

		private void LoadRegions()
		{
			using (var adp = new SqlDataAdapter("GetRegions", ConnStr))
			{
				adp.SelectCommand.CommandType = CommandType.StoredProcedure;
				var ds = new DataSet();
				adp.Fill(ds);
				this.cboRegions.DataSource = ds.Tables[0];
				this.cboRegions.ValueMember = "RegionId";
				this.cboRegions.DisplayMember = "RegionName";
			}
		}

		private void FindRegionPhotos()
		{
			this.lstPhotos.SelectedIndexChanged -= new System.EventHandler(this.lstPhotos_SelectedIndexChanged);

			int regionId = (int)this.cboRegions.SelectedValue;

			using (SqlDataAdapter adp = new SqlDataAdapter("GetRegionPhotos", ConnStr))
			{
				adp.SelectCommand.CommandType = CommandType.StoredProcedure;
				adp.SelectCommand.Parameters.AddWithValue("@RegionId", regionId);
				DataSet ds = new DataSet();
				adp.Fill(ds);
				this.lstPhotos.DataSource = ds.Tables[0];
				this.lstPhotos.ValueMember = "PhotoId";
				this.lstPhotos.DisplayMember = "Description";
			}
			this.lstPhotos.SelectedIndexChanged += new System.EventHandler(this.lstPhotos_SelectedIndexChanged);
			this.DisplayPhoto();
		}

		private void DisplayPhoto()
		{
			int photoId = (int)this.lstPhotos.SelectedValue;
			this.picImage.Image = this.GetPhoto(photoId);
		}

		private Image GetPhoto(int photoId)
		{
			Image photo;
			using (var ts = new TransactionScope())
			{
				using (SqlConnection conn = new SqlConnection(ConnStr))
				{
					conn.Open();

					string filePath;
					byte[] txnToken;

					using (SqlCommand cmd = new SqlCommand("GetPhotoForFilestream", conn))
					{
						cmd.CommandType = CommandType.StoredProcedure;
						cmd.Parameters.Add("@PhotoId", SqlDbType.Int).Value = photoId;

						using (SqlDataReader rdr = cmd.ExecuteReader(CommandBehavior.SingleRow))
						{
							rdr.Read();
							filePath = rdr.GetSqlString(0).Value;
							txnToken = rdr.GetSqlBinary(1).Value;
							rdr.Close();
						}
					}

					photo = this.LoadPhotoImage(filePath, txnToken);
				}
				ts.Complete();
			}

			return photo;
		}

		private Image LoadPhotoImage(string filePath, byte[] txnToken)
		{
			Image photo;

			using (var sfs = new SqlFileStream(filePath, txnToken, FileAccess.Read))
			{
				photo = Image.FromStream(sfs);
				sfs.Close();
			}

			return photo;
		}

	}
}